"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.asDataModel = void 0;
const errors_js_1 = require("../errors.js");
/**
 * Convert a CST message structure into its data model representation.
 *
 * @beta
 */
function asDataModel(msg) {
    for (const error of msg.errors)
        throw error;
    const declarations = msg.declarations.map(decl => ({
        name: asValue(decl.target).name,
        value: asExpression(decl.value)
    }));
    switch (msg.type) {
        case 'message':
            return {
                type: 'message',
                declarations,
                pattern: asPattern(msg.pattern)
            };
        case 'select':
            return {
                type: 'select',
                declarations,
                selectors: msg.selectors.map(asExpression),
                variants: msg.variants.map(cst => ({
                    keys: cst.keys.map(key => key.type === '*' ? { type: '*' } : asValue(key)),
                    value: asPattern(cst.value)
                }))
            };
        default:
            throw new errors_js_1.MessageSyntaxError('parse-error', 0, msg.source?.length ?? 0);
    }
}
exports.asDataModel = asDataModel;
function asPattern(cst) {
    const body = cst.body.map(el => el.type === 'text' ? { type: 'text', value: el.value } : asExpression(el));
    return { body };
}
function asExpression(cst) {
    if (cst.type !== 'expression') {
        throw new errors_js_1.MessageSyntaxError('parse-error', cst.start, cst.end);
    }
    let body;
    switch (cst.body.type) {
        case 'literal':
        case 'variable':
            body = asValue(cst.body);
            break;
        case 'function':
            body = asFunctionRef(cst.body);
            break;
        case 'reserved':
            body = asReserved(cst.body);
            break;
        default:
            throw new errors_js_1.MessageSyntaxError('parse-error', cst.start, cst.end);
    }
    return { type: 'expression', body };
}
function asFunctionRef(cst) {
    const fn = {
        type: 'function',
        kind: cst.kind,
        name: cst.name
    };
    if (cst.operand)
        fn.operand = asValue(cst.operand);
    if (cst.options && cst.options.length > 0) {
        fn.options = cst.options.map(opt => ({
            name: opt.name,
            value: asValue(opt.value)
        }));
    }
    return fn;
}
function asReserved(cst) {
    const res = {
        type: 'reserved',
        sigil: cst.sigil,
        source: cst.source
    };
    if (cst.operand)
        res.operand = asValue(cst.operand);
    return res;
}
function asValue(cst) {
    switch (cst.type) {
        case 'literal':
            return { type: 'literal', value: cst.value };
        case 'variable':
            return { type: 'variable', name: cst.name };
        default:
            throw new errors_js_1.MessageSyntaxError('parse-error', cst.start, cst.end);
    }
}
