"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseExpression = void 0;
const errors_js_1 = require("../errors.js");
const names_js_1 = require("./names.js");
const util_js_1 = require("./util.js");
const values_js_1 = require("./values.js");
// expression = "{" [s] (((literal / variable) [s annotation]) / annotation) [s] "}"
function parseExpression(ctx, start) {
    let pos = start + 1; // '{'
    pos += (0, util_js_1.whitespaces)(ctx.source, pos);
    const arg = ctx.source[pos] === '$'
        ? (0, values_js_1.parseVariable)(ctx, pos)
        : (0, values_js_1.parseLiteral)(ctx, pos, false);
    if (arg)
        pos = arg.end;
    let body;
    let junkError;
    pos += (0, util_js_1.whitespaces)(ctx.source, pos);
    switch (ctx.source[pos]) {
        case ':':
        case '+':
        case '-':
            body = parseFunctionRef(ctx, pos, arg);
            pos = body.end;
            break;
        case '!':
        case '@':
        case '#':
        case '%':
        case '^':
        case '&':
        case '*':
        case '<':
        case '>':
        case '?':
        case '~':
            body = parseReserved(ctx, pos, arg);
            pos = body.end;
            break;
        default:
            if (arg) {
                body = arg;
            }
            else {
                const source = ctx.source.substring(pos, pos + 1);
                body = { type: 'junk', start: pos, end: pos, source };
                junkError = new errors_js_1.MessageSyntaxError('parse-error', pos, pos + 1);
                ctx.errors.push(junkError);
            }
    }
    pos += (0, util_js_1.whitespaces)(ctx.source, pos);
    if (pos >= ctx.source.length) {
        ctx.onError('missing-char', pos, '}');
    }
    else if (ctx.source[pos] !== '}') {
        const errStart = pos;
        while (pos < ctx.source.length && ctx.source[pos] !== '}')
            pos += 1;
        if (body.type === 'junk') {
            body.end = pos;
            body.source = ctx.source.substring(body.start, pos);
            if (junkError)
                junkError.end = pos;
        }
        else {
            ctx.onError('extra-content', errStart, pos);
        }
    }
    else {
        pos += 1;
    }
    return { type: 'expression', start, end: pos, body };
}
exports.parseExpression = parseExpression;
// annotation = (function *(s option)) / reserved
// function = (":" / "+" / "-") name
function parseFunctionRef(ctx, start, operand) {
    const sigil = ctx.source[start];
    let pos = start + 1; // ':' | '+' | '-'
    const name = (0, names_js_1.parseNameValue)(ctx.source, pos);
    if (!name)
        ctx.onError('empty-token', pos, pos + 1);
    const options = [];
    pos += name.length;
    while (pos < ctx.source.length) {
        const ws = (0, util_js_1.whitespaces)(ctx.source, pos);
        if (ctx.source[pos + ws] === '}')
            break;
        if (ws === 0)
            ctx.onError('missing-char', pos, ' ');
        pos += ws;
        const opt = parseOption(ctx, pos);
        if (opt.end === pos)
            break; // error
        options.push(opt);
        pos = opt.end;
    }
    const kind = sigil === '+' ? 'open' : sigil === '-' ? 'close' : 'value';
    return { type: 'function', kind, operand, start, end: pos, name, options };
}
// option = name [s] "=" [s] (literal / variable)
function parseOption(ctx, start) {
    const name = (0, names_js_1.parseNameValue)(ctx.source, start);
    let pos = start + name.length;
    pos += (0, util_js_1.whitespaces)(ctx.source, pos);
    if (ctx.source[pos] === '=')
        pos += 1;
    else
        ctx.onError('missing-char', pos, '=');
    pos += (0, util_js_1.whitespaces)(ctx.source, pos);
    const value = ctx.source[pos] === '$'
        ? (0, values_js_1.parseVariable)(ctx, pos)
        : (0, values_js_1.parseLiteral)(ctx, pos, true);
    return { start, end: value.end, name, value };
}
// reserved       = reserved-start reserved-body
// reserved-start = "!" / "@" / "#" / "%" / "^" / "&" / "*" / "<" / ">" / "?" / "~"
// reserved-body  = *( [s] 1*(reserved-char / reserved-escape / literal))
// reserved-char  = %x00-08        ; omit HTAB and LF
//                / %x0B-0C        ; omit CR
//                / %x0E-19        ; omit SP
//                / %x21-5B        ; omit \
//                / %x5D-7A        ; omit { | }
//                / %x7E-D7FF      ; omit surrogates
//                / %xE000-10FFFF
function parseReserved(ctx, start, operand) {
    const sigil = ctx.source[start];
    let pos = start + 1; // sigil
    loop: while (pos < ctx.source.length) {
        const ch = ctx.source[pos];
        switch (ch) {
            case '\\': {
                switch (ctx.source[pos + 1]) {
                    case '\\':
                    case '{':
                    case '|':
                    case '}':
                        break;
                    default:
                        ctx.onError('bad-escape', pos, pos + 2);
                }
                pos += 2;
                break;
            }
            case '|':
                pos = (0, values_js_1.parseQuotedLiteral)(ctx, pos).end;
                break;
            case '}':
                break loop;
            default: {
                const cc = ch.charCodeAt(0);
                if (cc >= 0xd800 && cc < 0xe000) {
                    // surrogates are invalid here
                    ctx.onError('parse-error', pos, pos + 1);
                }
                pos += 1;
            }
        }
    }
    let prev = ctx.source[pos - 1];
    while (prev === '\r' || prev === '\n' || prev === '\t' || prev === ' ') {
        pos -= 1;
        prev = ctx.source[pos - 1];
    }
    const source = ctx.source.substring(start, pos);
    return { type: 'reserved', operand, sigil, source, start, end: pos };
}
