"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseMessage = exports.ParseContext = void 0;
const errors_js_1 = require("../errors.js");
const declarations_js_1 = require("./declarations.js");
const expression_js_1 = require("./expression.js");
const util_js_1 = require("./util.js");
const values_js_1 = require("./values.js");
class ParseContext {
    constructor(source, opt) {
        this.errors = [];
        this.resource = opt?.resource ?? false;
        this.source = source;
    }
    onError(type, start, end) {
        let err;
        switch (type) {
            case 'key-mismatch':
            case 'missing-fallback':
                err = new errors_js_1.MessageDataModelError(type, start, Number(end));
                break;
            case 'missing-char':
                err = new errors_js_1.MissingCharError(start, String(end));
                break;
            default:
                err = new errors_js_1.MessageSyntaxError(type, start, Number(end));
        }
        this.errors.push(err);
    }
}
exports.ParseContext = ParseContext;
// message = [s] *(declaration [s]) body [s]
// body = pattern / (selectors 1*([s] variant))
// selectors = match 1*([s] expression)
/**
 * Parse the string syntax representation of a message into
 * its corresponding CST representation.
 *
 * @beta
 */
function parseMessage(source, opt) {
    const ctx = new ParseContext(source, opt);
    const { declarations, end: pos } = (0, declarations_js_1.parseDeclarations)(ctx);
    if (source.startsWith('match', pos)) {
        return parseSelectMessage(ctx, pos, declarations);
    }
    else if (source[pos] === '{') {
        return parsePatternMessage(ctx, pos, declarations);
    }
    else {
        ctx.onError('parse-error', pos, source.length);
        return { type: 'junk', declarations, errors: ctx.errors, source };
    }
}
exports.parseMessage = parseMessage;
function parsePatternMessage(ctx, start, declarations) {
    const pattern = parsePattern(ctx, start);
    let pos = pattern.end;
    pos += (0, util_js_1.whitespaces)(ctx.source, pos);
    if (pos < ctx.source.length) {
        ctx.onError('extra-content', pos, ctx.source.length);
    }
    return { type: 'message', declarations, pattern, errors: ctx.errors };
}
function parseSelectMessage(ctx, start, declarations) {
    let pos = start + 5; // 'match'
    const match = { start, end: pos, value: 'match' };
    pos += (0, util_js_1.whitespaces)(ctx.source, pos);
    const selectors = [];
    while (ctx.source[pos] === '{') {
        const ph = (0, expression_js_1.parseExpression)(ctx, pos);
        switch (ph.body.type) {
            case 'function':
            case 'literal':
            case 'variable':
                break;
            default: {
                const { start, end } = ph.body;
                ctx.onError('bad-selector', start, end);
            }
        }
        selectors.push(ph);
        pos = ph.end;
        pos += (0, util_js_1.whitespaces)(ctx.source, pos);
    }
    if (selectors.length === 0) {
        ctx.onError('empty-token', pos, pos + 1);
    }
    const variants = [];
    pos += (0, util_js_1.whitespaces)(ctx.source, pos);
    while (ctx.source.startsWith('when', pos)) {
        const variant = parseVariant(ctx, pos, selectors.length);
        variants.push(variant);
        pos = variant.end;
        pos += (0, util_js_1.whitespaces)(ctx.source, pos);
    }
    if (pos < ctx.source.length) {
        ctx.onError('extra-content', pos, ctx.source.length);
    }
    return {
        type: 'select',
        declarations,
        match,
        selectors,
        variants,
        errors: ctx.errors
    };
}
// variant = when 1*(s key) [s] pattern
// key = literal / "*"
function parseVariant(ctx, start, selCount) {
    let pos = start + 4; // 'when'
    const when = { start, end: pos, value: 'when' };
    const keys = [];
    while (pos < ctx.source.length) {
        const ws = (0, util_js_1.whitespaces)(ctx.source, pos);
        pos += ws;
        const ch = ctx.source[pos];
        if (ch === '{')
            break;
        if (ws === 0)
            ctx.onError('missing-char', pos, ' ');
        const key = ch === '*'
            ? ({ type: '*', start: pos, end: pos + 1 })
            : (0, values_js_1.parseLiteral)(ctx, pos, true);
        if (key.end === pos)
            break; // error; reported in pattern.errors
        keys.push(key);
        pos = key.end;
    }
    if (selCount > 0 && keys.length !== selCount) {
        const end = keys.length === 0 ? pos : keys[keys.length - 1].end;
        ctx.onError('key-mismatch', start, end);
    }
    const value = parsePattern(ctx, pos);
    return { start, end: value.end, when, keys, value };
}
// pattern = "{" *(text / expression) "}"
function parsePattern(ctx, start) {
    if (ctx.source[start] !== '{') {
        ctx.onError('missing-char', start, '{');
        return { start, end: start, body: [] };
    }
    let pos = start + 1;
    const body = [];
    loop: while (pos < ctx.source.length) {
        switch (ctx.source[pos]) {
            case '{': {
                const ph = (0, expression_js_1.parseExpression)(ctx, pos);
                body.push(ph);
                pos = ph.end;
                break;
            }
            case '}':
                break loop;
            default: {
                const tx = (0, values_js_1.parseText)(ctx, pos);
                body.push(tx);
                pos = tx.end;
            }
        }
    }
    if (ctx.source[pos] === '}')
        pos += 1;
    else
        ctx.onError('missing-char', pos, '}');
    return { start, end: pos, body };
}
