"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveVariableRef = exports.getMessageValue = exports.lookupVariableRef = exports.isVariableRef = exports.UnresolvedExpression = void 0;
const errors_js_1 = require("../errors.js");
const index_js_1 = require("../runtime/index.js");
/**
 * Declarations aren't resolved until they're requierd,
 * and their resolution order matters for variable resolution.
 * This internal class is used to store any required data,
 * and to allow for `instanceof` detection.
 * @private
 */
class UnresolvedExpression {
    constructor(expression, scope) {
        this.expression = expression;
        this.scope = scope;
    }
}
exports.UnresolvedExpression = UnresolvedExpression;
/**
 * Type guard for {@link VariableRef} pattern elements
 *
 * @beta
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const isVariableRef = (part) => !!part && typeof part === 'object' && part.type === 'variable';
exports.isVariableRef = isVariableRef;
const isScope = (scope) => scope instanceof Object;
/**
 * Looks for the longest matching `.` delimited starting substring of name.
 * @returns `undefined` if value not found
 */
function getValue(scope, name) {
    if (isScope(scope)) {
        if (name in scope)
            return scope[name];
        const parts = name.split('.');
        for (let i = parts.length - 1; i > 0; --i) {
            const head = parts.slice(0, i).join('.');
            if (head in scope) {
                const tail = parts.slice(i).join('.');
                return getValue(scope[head], tail);
            }
        }
    }
    return undefined;
}
function lookupVariableRef(ctx, { name }) {
    const value = getValue(ctx.scope, name);
    if (value instanceof UnresolvedExpression) {
        const local = { ...ctx, scope: value.scope }.resolveExpression(value.expression);
        ctx.scope[name] = local;
        ctx.localVars.add(local);
        return local;
    }
    return value;
}
exports.lookupVariableRef = lookupVariableRef;
function getMessageValue(ctx, source, value) {
    let type = typeof value;
    if (type === 'object') {
        const mv = value;
        if (ctx.localVars.has(mv))
            return mv;
        if (value instanceof Number)
            type = 'number';
        else if (value instanceof String)
            type = 'string';
    }
    switch (type) {
        case 'bigint':
        case 'number': {
            const msgCtx = new index_js_1.MessageFunctionContext(ctx, source);
            return ctx.functions.number(msgCtx, {}, value);
        }
        case 'string': {
            const msgCtx = new index_js_1.MessageFunctionContext(ctx, source);
            return ctx.functions.string(msgCtx, {}, value);
        }
        default:
            return value === undefined ? undefined : (0, index_js_1.unknown)(source, value);
    }
}
exports.getMessageValue = getMessageValue;
function resolveVariableRef(ctx, ref) {
    const source = '$' + ref.name;
    const value = lookupVariableRef(ctx, ref);
    const mv = getMessageValue(ctx, source, value);
    if (mv !== undefined)
        return mv;
    const msg = `Variable not available: ${source}`;
    ctx.onError(new errors_js_1.MessageResolutionError('unresolved-var', msg, source));
    return (0, index_js_1.fallback)(source);
}
exports.resolveVariableRef = resolveVariableRef;
