"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.stringifyMessage = void 0;
const index_js_1 = require("../cst-parser/index.js");
const data_model_1 = require("../data-model");
const messageformat_1 = require("../messageformat");
const pattern_1 = require("../pattern");
const function_ref_1 = require("../pattern/function-ref");
/**
 * Stringify a message using its syntax representation.
 *
 * @beta
 */
function stringifyMessage(msg) {
    if (msg instanceof messageformat_1.MessageFormat)
        msg = msg.resolvedOptions().message;
    let res = '';
    for (const decl of msg.declarations)
        res += stringifyDeclaration(decl);
    if ((0, data_model_1.isPatternMessage)(msg)) {
        res += stringifyPattern(msg.pattern);
    }
    else if ((0, data_model_1.isSelectMessage)(msg)) {
        res += 'match';
        for (const sel of msg.selectors)
            res += ' ' + stringifyExpression(sel);
        for (const { keys, value } of msg.variants) {
            res += '\nwhen ';
            for (const key of keys) {
                res += ((0, pattern_1.isLiteral)(key) ? stringifyLiteral(key) : '*') + ' ';
            }
            res += stringifyPattern(value);
        }
    }
    return res;
}
exports.stringifyMessage = stringifyMessage;
function stringifyDeclaration({ name, value }) {
    return `let $${name} = ${stringifyExpression(value)}\n`;
}
function stringifyFunctionRef({ kind, name, operand, options }) {
    let res;
    switch (operand?.type) {
        case 'literal':
            res = stringifyLiteral(operand) + ' ';
            break;
        case 'variable':
            res = stringifyVariableRef(operand) + ' ';
            break;
        default:
            res = '';
    }
    res += (0, function_ref_1.functionRefSource)(kind, name);
    if (options)
        for (const opt of options)
            res += ' ' + stringifyOption(opt);
    return res;
}
function stringifyLiteral({ value }) {
    if ((0, index_js_1.isValidUnquotedLiteral)(value))
        return value;
    const esc = value.replace(/\\/g, '\\\\').replace(/\|/g, '\\|');
    return `|${esc}|`;
}
function stringifyOption({ name, value }) {
    const valueStr = (0, pattern_1.isVariableRef)(value)
        ? stringifyVariableRef(value)
        : stringifyLiteral(value);
    return `${name}=${valueStr}`;
}
function stringifyPattern({ body }) {
    let res = '';
    for (const el of body) {
        res += (0, pattern_1.isText)(el) ? el.value : stringifyExpression(el);
    }
    return `{${res}}`;
}
function stringifyExpression({ body }) {
    let res;
    switch (body.type) {
        case 'function':
            res = stringifyFunctionRef(body);
            break;
        case 'literal':
            res = stringifyLiteral(body);
            break;
        case 'variable':
            res = stringifyVariableRef(body);
            break;
        default:
            res = ''; // bad expression
    }
    return `{${res}}`;
}
function stringifyVariableRef(ref) {
    return '$' + ref.name;
}
